@@ -14,6 +14,7 @@ import sys
 import os
 import tempfile
 import subprocess
+import shutil
 import time
 import importlib.util
 import tracemalloc
@@ -666,7 +667,7 @@ MAIN_IFACE = 'universe.Ultimate'
 SYSTEM_BUS = False
 
 def load(mock, parameters):
-    mock.AddMethods(MAIN_IFACE, [('Answer', '', 'i', 'ret = 42')])
+    mock.AddMethods(MAIN_IFACE, [('Answer', 's', 'i', 'ret = 42')])
 ''')
             my_template.flush()
             (p_mock, dbus_ultimate) = self.spawn_server_template(
@@ -680,7 +681,18 @@ def load(mock, parameters):
             self.assertFalse(os.path.exists(my_template.name + 'c'))
             self.assertFalse(os.path.exists(importlib.util.cache_from_source(my_template.name)))
 
-        self.assertEqual(dbus_ultimate.Answer(), 42)
+        loop = GLib.MainLoop()
+        caught_signals = []
+
+        def method_called(method, args, **_):
+            caught_signals.append((method, args))
+            loop.quit()
+
+        dbus_mock = dbus.Interface(dbus_ultimate, dbusmock.MOCK_IFACE)
+        dbus_mock.connect_to_signal('MethodCalled', method_called)
+
+        self.assertEqual(dbus_ultimate.Answer("foo"), 42)
+        self.assertEqual(dbus_ultimate.Answer("bar"), 42)
 
         # should appear in introspection
         xml = dbus_ultimate.Introspect()
@@ -691,6 +703,23 @@ def load(mock, parameters):
         self.assertRaises(dbus.exceptions.DBusException,
                           dbus_ultimate.GetManagedObjects)
 
+        # Call should have been registered
+        mock_calls = dbus_mock.GetMethodCalls('Answer')
+        self.assertEqual(len(mock_calls), 2)
+        self.assertEqual(mock_calls[0][1], ['foo'])
+        self.assertEqual(mock_calls[1][1], ['bar'])
+
+        # Check signals
+        GLib.timeout_add(5000, loop.quit)
+        loop.run()
+
+        #  only one signal because we call loop.quit() in the handler
+        self.assertEqual(len(caught_signals), 1)
+        method, args = caught_signals[0]
+        self.assertEqual(method, 'Answer')
+        self.assertEqual(len(args), 1)
+        self.assertEqual(args[0], 'foo')
+
     def test_static_method(self):
         '''Static method in a template'''
 
@@ -705,9 +734,9 @@ def load(mock, parameters):
     pass
 
 @dbus.service.method(MAIN_IFACE,
-                     in_signature='',
+                     in_signature='s',
                      out_signature='i')
-def Answer(self):
+def Answer(self, string):
     return 42
 ''')
             my_template.flush()
@@ -717,13 +746,40 @@ def Answer(self):
             self.addCleanup(p_mock.terminate)
             self.addCleanup(p_mock.stdout.close)
 
-        self.assertEqual(dbus_ultimate.Answer(), 42)
+        loop = GLib.MainLoop()
+        caught_signals = []
 
+        def method_called(method, args, **_):
+            caught_signals.append((method, args))
+            loop.quit()
+
+        dbus_mock = dbus.Interface(dbus_ultimate, dbusmock.MOCK_IFACE)
+        dbus_mock.connect_to_signal('MethodCalled', method_called)
+
+        self.assertEqual(dbus_ultimate.Answer("foo"), 42)
+        self.assertEqual(dbus_ultimate.Answer("bar"), 42)
         # should appear in introspection
         xml = dbus_ultimate.Introspect()
         self.assertIn('<interface name="universe.Ultimate">', xml)
         self.assertIn('<method name="Answer">', xml)
 
+        # Call should have been registered
+        mock_calls = dbus_mock.GetMethodCalls('Answer')
+        self.assertEqual(len(mock_calls), 2)
+        self.assertEqual(mock_calls[0][1], ['foo'])
+        self.assertEqual(mock_calls[1][1], ['bar'])
+
+        # Check signals
+        GLib.timeout_add(5000, loop.quit)
+        loop.run()
+
+        #  only one signal because we call loop.quit() in the handler
+        self.assertEqual(len(caught_signals), 1)
+        method, args = caught_signals[0]
+        self.assertEqual(method, 'Answer')
+        self.assertEqual(len(args), 1)
+        self.assertEqual(args[0], 'foo')
+
     def test_local_nonexisting(self):
         self.assertRaises(ImportError, self.spawn_server_template, '/non/existing.py')
 
@@ -912,6 +968,89 @@ class TestSubclass(dbusmock.DBusTestCase):
         self.assertEqual(m.GetAll('org.test.MyMockI'), {'blurb': 5})
 
 
+class TestServiceAutostart(dbusmock.DBusTestCase):
+    '''Test service starting DBusMockObject'''
+
+    @classmethod
+    def setUpClass(cls):
+        cls.xdg_data_dir = tempfile.mkdtemp(prefix='dbusmock_xdg_')
+        cls.addClassCleanup(shutil.rmtree, cls.xdg_data_dir)
+
+        os.environ['XDG_DATA_DIRS'] = cls.xdg_data_dir
+
+        os.mkdir(os.path.join(cls.xdg_data_dir, 'dbus-1'))
+        system_dir = os.path.join(cls.xdg_data_dir, 'dbus-1', 'system-services')
+        session_dir = os.path.join(cls.xdg_data_dir, 'dbus-1', 'services')
+        os.mkdir(system_dir)
+        os.mkdir(session_dir)
+
+        with open(os.path.join(system_dir, 'org.TestSystem.service'), 'w', encoding='ascii') as s:
+            s.write('[D-BUS Service]\n' +
+                    'Name=org.TestSystem\n'
+                    'Exec=/usr/bin/python3 -c "import sys; from gi.repository import GLib, Gio; '
+                    '     Gio.bus_own_name(Gio.BusType.SYSTEM, \'org.TestSystem\', 0, None, None, lambda *args: sys.exit(0)); '
+                    '     GLib.MainLoop().run()"\n'
+                    'User=root')
+
+        with open(os.path.join(session_dir, 'org.TestSession.service'), 'w', encoding='ascii') as s:
+            s.write('[D-BUS Service]\n'
+                    'Name=org.TestSession\n'
+                    'Exec=/usr/bin/python3 -c "import sys; from gi.repository import GLib, Gio; '
+                    '     Gio.bus_own_name(Gio.BusType.SESSION, \'org.TestSession\', 0, None, None, lambda *args: sys.exit(0)); '
+                    '     GLib.MainLoop().run()"\n'
+                    'User=root')
+
+        cls.start_system_bus()
+        cls.start_session_bus()
+
+    def test_session_service_function_raise(self):
+        with self.assertRaises(AssertionError):
+            self.enable_service('does-not-exist')
+
+        with self.assertRaises(AssertionError):
+            self.disable_service('does-not-exist')
+
+    def test_session_service_isolation(self):
+        dbus_con = self.get_dbus(system_bus=False)
+        dbus_obj = dbus_con.get_object('org.freedesktop.DBus', '/org/freedesktop/DBus')
+        dbus_if = dbus.Interface(dbus_obj, 'org.freedesktop.DBus')
+
+        self.assertEqual(dbus_if.ListActivatableNames(), ['org.freedesktop.DBus'])
+        self.enable_service('org.TestSession')
+        self.addCleanup(self.disable_service, 'org.TestSession')
+        self.assertEqual(dbus_if.ListActivatableNames(), ['org.freedesktop.DBus', 'org.TestSession'])
+
+    def test_system_service_isolation(self):
+        dbus_con = self.get_dbus(system_bus=True)
+        dbus_obj = dbus_con.get_object('org.freedesktop.DBus', '/org/freedesktop/DBus')
+        dbus_if = dbus.Interface(dbus_obj, 'org.freedesktop.DBus')
+
+        self.assertEqual(dbus_if.ListActivatableNames(), ['org.freedesktop.DBus'])
+        self.enable_service('org.TestSystem', system_bus=True)
+        self.addCleanup(self.disable_service, 'org.TestSystem', system_bus=True)
+        self.assertEqual(dbus_if.ListActivatableNames(), ['org.freedesktop.DBus', 'org.TestSystem'])
+
+    def test_session_service_activation(self):
+        dbus_con = self.get_dbus(system_bus=False)
+        dbus_obj = dbus_con.get_object('org.freedesktop.DBus', '/org/freedesktop/DBus')
+        dbus_if = dbus.Interface(dbus_obj, 'org.freedesktop.DBus')
+
+        self.enable_service('org.TestSession')
+        self.addCleanup(self.disable_service, 'org.TestSession')
+
+        dbus_if.StartServiceByName('org.TestSession', 0)
+
+    def test_system_service_activation(self):
+        dbus_con = self.get_dbus(system_bus=True)
+        dbus_obj = dbus_con.get_object('org.freedesktop.DBus', '/org/freedesktop/DBus')
+        dbus_if = dbus.Interface(dbus_obj, 'org.freedesktop.DBus')
+
+        self.enable_service('org.TestSystem', system_bus=True)
+        self.addCleanup(self.disable_service, 'org.TestSystem', system_bus=True)
+
+        dbus_if.StartServiceByName('org.TestSystem', 0)
+
+
 if __name__ == '__main__':
     # avoid writing to stderr
     unittest.main(testRunner=unittest.TextTestRunner(stream=sys.stdout))
